from _Framework.MixerComponent import MixerComponent
from _Framework.SliderElement import SliderElement
from _Framework.ButtonElement import ButtonElement
from _Framework.InputControlElement import *
from MrChannelStripComponent import MrChannelStripComponent
from MrSysex import MrSysex, MrSysexParser
from const import *

class MrMixerComponent(MixerComponent):
	#Initializes the mixer component and creates the buttons and sliders
	def __init__(self,num_tracks):
		MixerComponent.__init__(self,num_tracks)
		self._strips = [ MrChannelStripComponent(i) for i in range(num_tracks) ]
		self._sliders = [SliderElement(1,0,(FADER + i)) for i in range(num_tracks) ]
		self._buttons = [ButtonElement(False,1,0,(BUTTON + i)) for i in range(num_tracks*2) ]
		self._strip_offset = 0
		self._assigned_tracks = []
		self.song().add_tracks_listener(self.__on_tracks_added_or_deleted)
		self._reassign_strips()
		return
	#This method inits all strips
	def _send_init(self):
		for tr in self._strips:
			tr._send_init()
	#This method reassigns all strips to the lemur faders and buttons and adds name listener
	def _reassign_strips(self):
		track_index = self._strip_offset
		for track in self._assigned_tracks:
		    if(track and track.name_has_listener(self._on_track_name_changed)):
		        track.remove_name_listener(self._on_track_name_changed)
		self._assigned_tracks = []
		all_tracks = (self.song().tracks + self.song().return_tracks)
		count = 0
		for strip in self._strips:
			btc = count * 2
			if(track_index < len(all_tracks)):
				track = all_tracks[track_index]
				strip.set_invert_mute_feedback(True)
				strip.set_track(track)
				strip.set_volume_control(self._sliders[count])
				strip.set_solo_button(self._buttons[btc])
				strip.set_mute_button(self._buttons[btc+1])
				track.add_name_listener(self._on_track_name_changed)
				self._assigned_tracks.append(track)
			else:
				strip.set_track(None)
				strip.set_volume_control(None)
				self._sliders[count].send_value(0,True)
				strip.set_solo_button(None)
				strip.set_mute_button(None)
			track_index += 1
			count += 1
		self._send_init()
	#Receives the sysex from MrLemur.py and reads the messages
	def handle_sysex(self,sysex):
	    if(sysex[1]== BANK_RIGHT):
	        self._strip_offset += 8
	        self.__validate_strip_offset()
	    elif(sysex[1]==BANK_LEFT):
	        self._strip_offset -= 8
	        self.__validate_strip_offset()
	    self._reassign_strips()
	#This method is called when a track changes it's name
	def _on_track_name_changed(self):
	    self._reassign_strips()
	#This method checks if the offset is within available tracks limit    
	def __validate_strip_offset(self):
	    all_tracks = (self.song().tracks + self.song().return_tracks)
	    self._strip_offset = min(self._strip_offset, (len(all_tracks)-1))
	    self._strip_offset = max(0,self._strip_offset)
	#This method is called when a track is added or deleted
	def __on_tracks_added_or_deleted(self):
	    self.__validate_strip_offset()
	    self._reassign_strips()
        
